@@ -139,6 +139,10 @@ class MandrillPayload(RequestsPayload):
             "content": attachment.b64content
         })
 
+    def set_envelope_sender(self, email):
+        # Only the domain is used
+        self.data["message"]["return_path_domain"] = email.domain
+
     def set_metadata(self, metadata):
         self.data["message"]["metadata"] = metadata
 
@@ -252,9 +256,9 @@ class MandrillPayload(RequestsPayload):
         ('template_content', combine, None),
     )
 
-    def set_async(self, async):
+    def set_async(self, is_async):
         self.deprecated_to_esp_extra('async')
-        self.esp_extra['async'] = async
+        self.esp_extra['async'] = is_async
 
     def set_ip_pool(self, ip_pool):
         self.deprecated_to_esp_extra('ip_pool')
@@ -268,6 +272,10 @@ class MandrillPayload(RequestsPayload):
         self.deprecation_warning('message.merge_vars', 'message.merge_data')
         self.set_merge_data(merge_vars)
 
+    def set_return_path_domain(self, domain):
+        self.deprecation_warning('message.return_path_domain', 'message.envelope_sender')
+        self.esp_extra.setdefault('message', {})['return_path_domain'] = domain
+
     def set_template_name(self, template_name):
         self.deprecation_warning('message.template_name', 'message.template_id')
         self.set_template_id(template_name)
