@@ -1,44 +1,13 @@
 from base64 import b64decode
-from email import message_from_string
 from email.message import Message
 from email.utils import unquote
 
 import six
 from django.core.files.uploadedfile import SimpleUploadedFile
 
+from ._email_compat import EmailParser, EmailBytesParser
 from .utils import angle_wrap, get_content_disposition, parse_address_list, parse_rfc2822date
 
-# Python 2/3.*-compatible email.parser.HeaderParser(policy=email.policy.default)
-try:
-    # With Python 3.3+ (email6) package, can use HeaderParser with default policy
-    from email.parser import HeaderParser
-    from email.policy import default as accurate_header_unfolding_policy  # vs. compat32
-
-except ImportError:
-    # Earlier Pythons don't have HeaderParser, and/or try preserve earlier compatibility bugs
-    # by failing to properly unfold headers (see RFC 5322 section 2.2.3)
-    from email.parser import Parser
-    import re
-    accurate_header_unfolding_policy = object()
-
-    class HeaderParser(Parser, object):
-        def __init__(self, _class, policy=None):
-            # This "backport" doesn't actually support policies, but we want to ensure
-            # that callers aren't trying to use HeaderParser's default compat32 policy
-            # (which doesn't properly unfold headers)
-            assert policy is accurate_header_unfolding_policy
-            super(HeaderParser, self).__init__(_class)
-
-        def parsestr(self, text, headersonly=True):
-            unfolded = self._unfold_headers(text)
-            return super(HeaderParser, self).parsestr(unfolded, headersonly=True)
-
-        @staticmethod
-        def _unfold_headers(text):
-            # "Unfolding is accomplished by simply removing any CRLF that is immediately followed by WSP"
-            # (WSP is space or tab, and per email.parser semantics, we allow CRLF, CR, or LF endings)
-            return re.sub(r'(\r\n|\r|\n)(?=[ \t])', "", text)
-
 
 class AnymailInboundMessage(Message, object):  # `object` ensures new-style class in Python 2)
     """
@@ -153,9 +122,7 @@ class AnymailInboundMessage(Message, object):  # `object` ensures new-style clas
         # should themselves be AnymailInboundMessage.
         for part in self.walk():
             if part.get_content_type() == content_type and not part.is_attachment():
-                payload = part.get_payload(decode=True)
-                if payload is not None:
-                    return payload.decode('utf-8')
+                return part.get_content_text()
         return None
 
     # Backport from Python 3.5 email.message.Message
@@ -192,7 +159,7 @@ class AnymailInboundMessage(Message, object):  # `object` ensures new-style clas
                              "(perhaps you want as_bytes()?)")
         return self.get_payload(decode=True)
 
-    def get_content_text(self, charset='utf-8'):
+    def get_content_text(self, charset=None, errors=None):
         """Return the payload decoded to text"""
         maintype = self.get_content_maintype()
         if maintype == 'message':
@@ -206,7 +173,13 @@ class AnymailInboundMessage(Message, object):  # `object` ensures new-style clas
             # and it's not clear which one is the "content".
             raise ValueError("get_content_text() is not valid on multipart messages "
                              "(perhaps you want as_string()?)")
-        return self.get_payload(decode=True).decode(charset)
+        else:
+            payload = self.get_payload(decode=True)
+            if payload is None:
+                return payload
+            charset = charset or self.get_content_charset('US-ASCII')
+            errors = errors or 'replace'
+            return payload.decode(charset, errors=errors)
 
     def as_uploaded_file(self):
         """Return the attachment converted to a Django UploadedFile"""
@@ -226,7 +199,24 @@ class AnymailInboundMessage(Message, object):  # `object` ensures new-style clas
     @classmethod
     def parse_raw_mime(cls, s):
         """Returns a new AnymailInboundMessage parsed from str s"""
-        return message_from_string(s, cls)
+        if isinstance(s, six.text_type):
+            # Avoid Python 3.x issue https://bugs.python.org/issue18271
+            # (See test_inbound: test_parse_raw_mime_8bit_utf8)
+            return cls.parse_raw_mime_bytes(s.encode('utf-8'))
+        return EmailParser(cls).parsestr(s)
+
+    @classmethod
+    def parse_raw_mime_bytes(cls, b):
+        """Returns a new AnymailInboundMessage parsed from bytes b"""
+        return EmailBytesParser(cls).parsebytes(b)
+
+    @classmethod
+    def parse_raw_mime_file(cls, fp):
+        """Returns a new AnymailInboundMessage parsed from file-like object fp"""
+        if isinstance(fp.read(0), six.binary_type):
+            return EmailBytesParser(cls).parse(fp)
+        else:
+            return EmailParser(cls).parse(fp)
 
     @classmethod
     def construct(cls, raw_headers=None, from_email=None, to=None, cc=None, subject=None, headers=None,
@@ -252,7 +242,7 @@ class AnymailInboundMessage(Message, object):  # `object` ensures new-style clas
         :return: {AnymailInboundMessage}
         """
         if raw_headers is not None:
-            msg = HeaderParser(cls, policy=accurate_header_unfolding_policy).parsestr(raw_headers)
+            msg = EmailParser(cls).parsestr(raw_headers, headersonly=True)
             msg.set_payload(None)  # headersonly forces an empty string payload, which breaks things later
         else:
             msg = cls()
