@@ -59,6 +59,10 @@ def attach_inline_image_file(message, path, subtype=None, idstring="img", domain
 
 def attach_inline_image(message, content, filename=None, subtype=None, idstring="img", domain=None):
     """Add inline image to an EmailMessage, and return its content id"""
+    if domain is None:
+        # Avoid defaulting to hostname that might end in '.com', because some ESPs
+        # use Content-ID as filename, and Gmail blocks filenames ending in '.com'.
+        domain = 'inline'  # valid domain for a msgid; will never be a real TLD
     content_id = make_msgid(idstring, domain)  # Content ID per RFC 2045 section 7 (with <...>)
     image = MIMEImage(content, subtype)
     image.add_header('Content-Disposition', 'inline', filename=filename)
