@@ -1,7 +1,6 @@
 import base64
 import mimetypes
 from base64 import b64encode
-from collections import Mapping, MutableMapping
 from datetime import datetime
 from email.mime.base import MIMEBase
 from email.utils import formatdate, getaddresses, unquote
@@ -15,6 +14,11 @@ from django.utils.functional import Promise
 from django.utils.timezone import utc, get_fixed_timezone
 from six.moves.urllib.parse import urlsplit, urlunsplit
 
+try:
+    from collections.abc import Mapping, MutableMapping  # Python 3.3+
+except ImportError:
+    from collections import Mapping, MutableMapping
+
 from .exceptions import AnymailConfigurationError, AnymailInvalidAddress
 
 
@@ -287,7 +291,8 @@ class Attachment(object):
                     self.content = attachment.as_string().encode(self.encoding)
             self.mimetype = attachment.get_content_type()
 
-            if get_content_disposition(attachment) == 'inline':
+            content_disposition = get_content_disposition(attachment)
+            if content_disposition == 'inline' or (not content_disposition and 'Content-ID' in attachment):
                 self.inline = True
                 self.content_id = attachment["Content-ID"]  # probably including the <...>
                 if self.content_id is not None:
